from gem.embedding.gf import GraphFactorization as gf
from gem.evaluation import evaluate_graph_reconstruction as gr
from gem.utils import graph_util
from gem.embedding.hope import HOPE as hp
import numpy as np
import scipy.spatial.distance as sc
import optparse
import matplotlib.pyplot as plt
from gem.utils import graph_util, plot_util
from gem.evaluation import visualize_embedding as viz
import networkx as nx


class CFGEmbedding:

	def embedding_distances(self, CFGFile, emb_size, outDistFile):
		
		fileID = open(CFGFile, 'r')
		
		result_array = np.empty((0, emb_size))
		
		i = 0;
		for line in fileID:
			i = i+1;
			print 'Line: %d' % (i)
			#Building graph
			edges = line.rstrip(';\n\r').split(';')
			G = nx.DiGraph()
			for edgeLine in edges:
				edge = edgeLine.strip().split()
				G.add_edge(int(edge[0]), int(edge[1]), weight=1.0)
			print 'Num nodes: %d, num edges: %d' % (G.number_of_nodes(), G.number_of_edges())
			
			#Init embedding learner
			em = hp(emb_size, 0.01)
			
			#Learn embeddings
			try:
				Y, t = em.learn_embedding(G, edge_f=None, is_weighted=True, no_python=True)
			except:
				print 'Exception!'
				Y = np.random.rand(3,emb_size)
				pass
				
			#Mean Pooling
			Y1 = Y.mean(axis=0);
			result_array = np.append(result_array, [Y1], axis=0)
			
		#Compute distances
		Z = sc.squareform(sc.pdist(result_array, 'euclidean'))

		#Export distances
		np.savetxt(outDistFile, Z, delimiter=",")



if __name__ == '__main__':
	
	#Read inputs
	parser = optparse.OptionParser()
	parser.add_option('--cfg', help='/path/to/<granularity>.cfg')
	parser.add_option('--emb', type="int", help='Embedding size')
	parser.add_option('--out', help='/path/to/<granularity>.dist')
	(opt, args) = parser.parse_args()

	#Learn embedding and compute distances
	cfg_embedding = CFGEmbedding()
	cfg_embedding.embedding_distances(opt.cfg, opt.emb, opt.out)
